
#include "stdafx.h"
#include "csplib/constraint_network.hpp"
#include "csplib/adb/adbio.hpp"

size_t maxScope(ConstraintNetwork &cn)
{
  size_t maxsize = 0;
  ConstraintNetwork::relcol_t rels = cn.getRelations();
  
  for(ConstraintNetwork::relcol_t::const_iterator it = rels.begin(); it != rels.end(); ++it)
    {
    const Relation &r = *(*it);
    
    if (r.getNumColumns() > maxsize)
      maxsize = r.getNumColumns();
    }
  return maxsize;
}

size_t maxRowCount(ConstraintNetwork &cn)
{
  size_t maxsize = 0;
  ConstraintNetwork::relcol_t rels = cn.getRelations();
  
  for(ConstraintNetwork::relcol_t::const_iterator it = rels.begin(); it != rels.end(); ++it)
    {
    const Relation &r = *(*it);
    
    if (r.getNumRows() > maxsize)
      maxsize = r.getNumRows();
    }
  return maxsize;
}

double maxNumTuples(ConstraintNetwork &cn)
{
  double maxsize = 0;
  ConstraintNetwork::relcol_t rels = cn.getRelations();
  
  for(ConstraintNetwork::relcol_t::const_iterator it = rels.begin(); it != rels.end(); ++it)
    {
    const Relation &r = *(*it);
    if (r.getNumExpandedTuples() > maxsize)
      maxsize = r.getNumExpandedTuples();
    }
  return maxsize;
}

double sizeUncomp(ConstraintNetwork &cn)
{
  double size = 0;
  ConstraintNetwork::relcol_t rels = cn.getRelations();
  
  for(ConstraintNetwork::relcol_t::const_iterator it = rels.begin(); it != rels.end(); ++it)
    {
    const Relation &r = *(*it);
    
    size += r.getNumExpandedTuples() * r.getNumColumns();
    }
  return size;
}
long double searchSpace(ConstraintNetwork &cn)
{
  long double ss = 1.0;

  for(size_t i = 0; i < cn.getNumVariables();i++)
    {
    Variable &v = cn.getVariable(i);
    ss *= (double)v.getDomainSize(); 
    }
  
  return ss;
}

size_t maxDomSize(ConstraintNetwork &cn)
{
  size_t maxsize = 0;
  
  for(size_t i = 0; i < cn.getNumVariables();i++)
    {
    Variable &v = cn.getVariable(i);
    if (v.getDomainSize() > maxsize)
      maxsize = v.getDomainSize();
    }
  return maxsize;
}
    
template<typename T>
std::string number2latex(T d)
{
  std::stringstream ss;

  ss.setf (std::ios_base::scientific, std::ios_base::floatfield);
  ss.precision(2);
  ss << d;
  
  std::string s = ss.str();
  
  size_t pos  = s.find("e+");
  if (pos != std::string::npos)
    {
    size_t pos2 = s.find_first_not_of('0',pos+2);
    if (pos2 != std::string::npos && pos2 != (pos+2))
      s.erase(pos+2, pos2-pos-2);
    s.replace(pos, 2, "\\times 10^{");
    s.append("}");
    }
  
  if (s == "inf")
    s = "\\dagger";

  return s;
}

int main(int argc, char* argv[])
{
  ConstraintNetwork             instance;
  
  ADB::importADBFile(instance, argv[1]);

  char *pos = strrchr(argv[1],'/');
  if (pos == 0)
    pos = argv[1];
  else
    pos++;
  char *extpos = strrchr(pos, '.');
  char name[256];
  
  if (extpos == 0)
    strcpy(name, pos);
  else
    {
    strncpy(name, pos, extpos - pos);
    name[extpos-pos] = '\0';
    }
  
    
  size_t numvars = instance.getNumVariables();
  size_t numrels = instance.getNumRelations();
  size_t maxscope = maxScope(instance);
  size_t maxdom = maxDomSize(instance);
  long double searchspace = searchSpace(instance);
  double uncomptup = maxNumTuples(instance);
  double uncompsize = sizeUncomp(instance);
  
  size_t compsize = (size_t)instance.getRelSize();
  size_t comptup = maxRowCount(instance);
  
  
  
  std::cout << "#Variables               : " << numvars << "\n"
            << "#Constraints             : " << numrels << "\n"
            << "Largest scope            : " << maxscope  << "\n"
            << "Max domain               : " << maxdom << "\n"
            << "Search space             : " << searchspace << "\n"
            << "Uncompressed size        : " << uncompsize << "\n"
            << "Uncompressed max tuples  : " << uncomptup << "\n";

  std::cout << "Compressed size          : " << compsize << "\n"
            << "Compressed max tuples    : " << comptup << "\n";

  std::cout << "Latex: " << name << " & $"
            << numvars << "$ & $" << numrels << "$ & $" << maxscope << "$ & $"
            << maxdom << "$ & $" << number2latex(searchspace) << "$ & $" << number2latex(uncompsize) << "$ & $"
            << number2latex(uncomptup) << "$ & $" << compsize << "$ & $" << comptup << "$\\\\\n";
  
}

