/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef COMMON_H
#define COMMON_H

#define __WIN32__
#define _WIN32_WINNT 0x0500 

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers

#define NOMINMAX
#define _STLP_NEW_PLATFORM_SDK
//#define _STLP_NO_EXTENSIONS

#ifdef NDEBUG
# undef _STLP_DEBUG
#else
# define _STLP_DEBUG                1
# define _STLP_DEBUG_UNINITIALIZED  1
//# define _STLP_DEBUG_ALLOC
//# define _STLP_USE_MALLOC 
#endif

#ifdef _MSC_VER
#pragma warning( disable : 4786 4995 4511 4512 4267 4503)

#pragma warning( push )
#pragma warning( disable : 4311 4702 4127 4267)
#endif

#include <windows.h>
#include <cstdlib>
#include <stdexcept>
#include <iostream>
#include <string>
#include <vector>
#include <deque>
#include <set>
#include <list>
#include <algorithm>
#include <functional>
#include <iterator>
#include <cassert>
#include <sstream>
#include "stlutil.h"
#include <boost/utility.hpp>
#include <boost/smart_ptr.hpp>
#include <boost/pool/object_pool.hpp>
#include <boost/static_assert.hpp>
#include <boost/type_traits.hpp>
#include <boost/lexical_cast.hpp> 

#ifdef _MSC_VER
#pragma warning( pop )
#endif

#ifdef NDEBUG
#define DEBUG(x)
#else
#define DEBUG(x) x
#endif

struct internal_error : public std::runtime_error
  {
  internal_error(const std::string &file, unsigned int line)
    : std::runtime_error("Internal error in file " + file + " line " + 
                         boost::lexical_cast<std::string>(line)) {}
  };
#define INTERNAL_ERROR  throw internal_error(__FILE__, __LINE__)

// Vector of indicies
typedef std::vector<size_t> idvector_t;

// Pair of indicies
typedef std::pair<size_t, size_t> idpair_t;

typedef std::vector<bool> boolvector_t;


template<typename T>
std::ostream &write_set(std::ostream &s, const T &c)
  {
  s << '{';
  typename T::const_iterator it = c.begin();
  while (it != c.end())
    {
    if (it != c.begin())
      s << ",";
    s << (*it++);
    }
  s << '}';
  return s;
  }

#endif
