/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
template<size_t N>
class BitmapCompressedColumn : public ColumnBase<CompressedColumn, BitsetCellTraits<NewBitset<N> >,
                                                 UncompressedColumn, ScalarCellTraits<unsigned char> >
  {
  typedef ConcreteUncompressedColumn<unsigned char,BitsetCellTraits<NewBitset<N> > > uncompressed_t;
  typedef typename NewBitset<N>::index_type index_type;

  public:
    BitmapCompressedColumn(Variable &var, size_t size)
      : ColumnBase<CompressedColumn, 
                   BitsetCellTraits<NewBitset<N> >,
                   UncompressedColumn, ScalarCellTraits<unsigned char> >(var) 
      {
      if (size > 0)
        container.reserve(size);
      }

    BitmapCompressedColumn(Variable &var, const urvector_t &ur)
      : ColumnBase<CompressedColumn, 
                    BitsetCellTraits<NewBitset<N> >,
                    UncompressedColumn, ScalarCellTraits<unsigned char> >(var) 
      {
      urvector_t::const_iterator it = ur.begin(), pend = ur.end();

      while (it != pend)
        {
        NewBitset<N>  bs;
        idvector_t::const_iterator bit = it->second->begin(), pbend = it->second->end();
        while(bit != pbend)
          bs.set((*bit++));
        container.push_back(bs);
        ++it;
        }
      }

    virtual void addCell(const idvector_t &values)
      {
      NewBitset<N>  bs;
      idvector_t::const_iterator bit = values.begin(), pbend = values.end();
      while(bit != pbend)
        bs.set(static_cast<index_type>(*bit++));
      container.push_back(bs);
      }

    virtual void addCellCopy(size_t copy_from)
      {
      container.push_back(container[copy_from]);
      }

    virtual Column *cloneFromJoinIndex(const joinindex_t &ji, bool useFirst) const
      {
      BitmapCompressedColumn *newCol = new BitmapCompressedColumn(m_Variable, ji.size());
      container.cloneFromJoinIndex(ji, newCol->container, useFirst);
      return newCol;
      }

    virtual Column *clone(size_t numNewCells, bool interleave) const
      {
      if (numNewCells == 0)
        numNewCells = container.size();

      BitmapCompressedColumn *newCol = new BitmapCompressedColumn(m_Variable, numNewCells);
      container.clone(numNewCells, newCol->container, interleave);
      return newCol;
      }

    virtual Column *createFromJoinIndex(const joinindex_t &ji, const Column &col) const
      {
      if (col.isCompressed())
        {
        BitmapCompressedColumn *newCol = new BitmapCompressedColumn(m_Variable, ji.size());
        container.createFromJoinIndex(ji, get_container_c(col), newCol->container);
        return newCol;
        }
      else
        {
        BitmapCompressedColumn::uncompressed_t *newCol = new BitmapCompressedColumn::uncompressed_t(m_Variable, ji.size());
        container.createFromJoinIndex(ji, get_container_u(col), newCol->container);
        return newCol;
        }
      }
  };
