/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// Compressed column based on vector of vector of T
template<typename T>
class ConcreteCompressedColumn : public ColumnBase<CompressedColumn, ContainerCellTraits<T>,
                                                   UncompressedColumn, ScalarCellTraits<T> >
  {
  public:
    ConcreteCompressedColumn(Variable &var, size_t size)
      : ColumnBase<CompressedColumn, ContainerCellTraits<T>,
                   UncompressedColumn, ScalarCellTraits<T> >(var) 
      {
      if (size > 0)
        container.reserve(size);
      }

    ConcreteCompressedColumn(Variable &var, const urvector_t &ur)
      : ColumnBase<CompressedColumn, ContainerCellTraits<T>,
                   UncompressedColumn, ScalarCellTraits<T> >(var) 
      {
      container.reserve(ur.size());
      urvector_t::const_iterator it = ur.begin(), pend = ur.end();

      while (it != pend)
        {
        idvector_t *idv = it->second;
        std::sort(idv->begin(), idv->end());
        idvector_t::const_iterator uit = idv->begin(), newend = std::unique(idv->begin(), idv->end());
        std::vector<T> *cell = new std::vector<T>;

        while(uit != newend)
          {
          cell->push_back(T(*uit++));
          }
        container.push_back(cell);
        ++it;
        }
      }

    void addCell(const idvector_t &values)
      {
      idvector_t idv(values);
      std::sort(idv.begin(), idv.end());
      idvector_t::const_iterator uit = idv.begin(), newend = std::unique(idv.begin(), idv.end());
      std::vector<T> *cell = new std::vector<T>;
      while(uit != newend)
        {
        cell->push_back(T(*uit++));
        }
      container.push_back(cell);
      }

    void addCellCopy(size_t copy_from)
      {
      std::vector<T> *cell = new std::vector<T>(*container[copy_from]);
      container.push_back(cell);
      }

    ~ConcreteCompressedColumn()
      {
      for_all(container, delete_element<std::vector<T> >());
      }

    virtual Column *clone(size_t numNewCells, bool interleave) const
      {
      if (numNewCells == 0)
        numNewCells = container.size();

      ConcreteCompressedColumn *newCol = new ConcreteCompressedColumn(m_Variable, numNewCells);
      container.clone(numNewCells, newCol->container, interleave);
      return newCol;
      }


    virtual Column *cloneFromJoinIndex(const joinindex_t &ji, bool useFirst) const
      {
      ConcreteCompressedColumn *newCol = new ConcreteCompressedColumn(m_Variable, ji.size());
      container.cloneFromJoinIndex(ji, newCol->container, useFirst);
      return newCol;
      }

    virtual Column *createFromJoinIndex(const joinindex_t &ji, const Column &col) const
      {
      if (col.isCompressed())
        {
        ConcreteCompressedColumn *newCol = new ConcreteCompressedColumn(m_Variable, ji.size());
        container.createFromJoinIndex(ji, get_container_c(col), newCol->container);
        return newCol;
        }
      else
        {
        ConcreteUncompressedColumn<T> *newCol = new ConcreteUncompressedColumn<T>(m_Variable, ji.size());
        container.createFromJoinIndex(ji, get_container_u(col), newCol->container);
        return newCol;
        }
      }
  };
