/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
template <typename T, 
          typename CompressedTraits = ContainerCellTraits<T> >
class ConcreteUncompressedColumn : public ColumnBase<UncompressedColumn,
                                                     ScalarCellTraits<T>,
                                                     CompressedColumn,
                                                     CompressedTraits>
  {
  public:
    ConcreteUncompressedColumn(Variable &var, size_t size = 0)
      : ColumnBase<UncompressedColumn, ScalarCellTraits<T>, 
                   CompressedColumn, CompressedTraits>(var) 
      {
      if (size > 0) 
        container.reserve(size);
      }

    virtual void addCell(size_t index)
      {
      if (index >= m_Variable.getDomainSize())
        {
        std::cout << "index=" << index << ",domsize=" << m_Variable.getDomainSize() << ",var=" << m_Variable.getName() << "\n";
        throw std::domain_error("Invalid domain index");
        }
      container.push_back(T(index));
      }

    virtual void setCell(size_t cellno, size_t index)
      {
      if (index >= container.capacity())
        throw std::domain_error("Invalid cell number");
      if (cellno >= container.size())
        container.resize(cellno+1);
      container[cellno] = T(index);
      }

    virtual size_t getCell(size_t index) const
      {
      return size_t(container[index]);
      }
    
    virtual void addCellCopy(size_t index)
      {
      container.push_back(container[index]);
      }
    
    virtual Column *clone(size_t numNewCells, bool interleave) const
      {
      if (numNewCells == 0)
        numNewCells = container.size();

      ConcreteUncompressedColumn *newCol = new ConcreteUncompressedColumn(m_Variable, numNewCells);
      container.clone(numNewCells, newCol->container, interleave);
      return newCol;
      }

    virtual Column *cloneFromJoinIndex(const joinindex_t &ji, bool useFirst) const
      {
      ConcreteUncompressedColumn *newCol = new ConcreteUncompressedColumn(m_Variable, ji.size());
      container.cloneFromJoinIndex(ji, newCol->container, useFirst);
      return newCol;
      }

    virtual Column *createFromJoinIndex(const joinindex_t &ji, const Column &col) const
      {
      if (col.isCompressed())
        {
        ConcreteUncompressedColumn *newCol = new ConcreteUncompressedColumn(m_Variable, ji.size());
        container.createFromJoinIndex(ji, get_container_c(col), newCol->container);
        return newCol;
        }
      else
        {
        // If no columns are compressed, just clone 
        return cloneFromJoinIndex(ji, true);
        }
      }
  };
