/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef GRAPH_H
#define GRAPH_H
#include <boost/graph/adjacency_list.hpp>
#include "constraint_network.hpp"

// Custom Vertex Property for storing pointer to relation
struct vertex_relation_t 
  {
  typedef boost::vertex_property_tag kind;
  };


// Custom Vertex Property for storing pointer to variable
struct vertex_variable_t 
  {
  typedef boost::vertex_property_tag kind;
  };

struct edge_variables_t 
  {
  typedef boost::edge_property_tag kind;
  };

// Vertex properties for dual graph
typedef boost::property<boost::vertex_name_t, size_t,
                        boost::property<boost::vertex_color_t, boost::default_color_type,
                        boost::property<vertex_relation_t, Relation *> > > DualVertexProperty;

// Edge properties for dual graph
typedef boost::property<boost::edge_name_t, std::string, 
                        boost::property<edge_variables_t, varset_t,
                        boost::property<boost::edge_color_t, boost::default_color_type,
                        boost::property<boost::edge_weight_t, int> > > > DualEdgeProperty;

// The dual graph
typedef boost::adjacency_list<boost::setS, boost::vecS, boost::undirectedS,
                              DualVertexProperty, DualEdgeProperty> dualgraph_t;

// Vertex properties for primal graph
typedef boost::property<boost::vertex_name_t, std::string,
                        boost::property<boost::vertex_color_t, boost::default_color_type,
                        boost::property<vertex_variable_t, Variable *> > > PrimalVertexProperty;

// Edge properties for dual graph
typedef boost::property<boost::edge_name_t, std::string, 
                        boost::property<boost::edge_color_t, boost::default_color_type,
                        boost::property<boost::edge_weight_t, size_t> > > PrimalEdgeProperty;

// The primal graph
typedef boost::adjacency_list<boost::setS, boost::vecS, boost::undirectedS, 
                              PrimalVertexProperty, PrimalEdgeProperty> primalgraph_t;

void createDualGraph(ConstraintNetwork &csp, dualgraph_t &g);
void createPrimalGraph(ConstraintNetwork &csp, primalgraph_t &g);
void create_join_tree(dualgraph_t &g);

void check_rip(const dualgraph_t &g); 
void check_chordality(const primalgraph_t &G);
bool is_cyclic(dualgraph_t &g);

bool directional_arc_consistency(dualgraph_t &dg, size_t max_rel_size, Relation *root = 0);

// Output graph in dot format
std::ostream &operator<<(std::ostream &s, const dualgraph_t &g);
std::ostream &operator<<(std::ostream &s, const primalgraph_t &g);
#endif
