/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "stdafx.h"
#include <fstream>
#include "column.hpp"
#include "uniquerows.hpp"

const size_t primes[] = {2, 5, 11, 23, 47, 97, 197, 397, 797, 1597,
                         3203, 6421, 12853, 25717, 51437, 102877, 205759,
                         411527, 823117, 1646237, 3292489, 6584983,
                         13169977, 26339969, 52679969, 105359939,
                         210719881, 421439783, 842879579, 1685759167};

const size_t NUM_PRIMES = sizeof(primes)/sizeof(primes[0]);

// Return the number of buckets needed to hold the specified
// number of entries
size_t calculateBucketCount(size_t numRows)
  {
  size_t first = 0, last = NUM_PRIMES - 1;
  while (first < last) 
    {
    size_t mid = (first + last) / 2;
    if(numRows < primes[mid]) 
      last = mid;
    else 
      first = mid + 1;
    }
  return primes[last];
  }

UniqueComplement::UniqueComplement(const colvector_t &columns,
                       colvector_t::const_iterator colmask)
    : base_t(calculateBucketCount((*colmask)->getNumCells()),
             Hasher(values), detail::ColumnComplementEq(columns, colmask)),
    values(columns, colmask)
  {
  }

UniqueRows::UniqueRows(const colvector_t &columns)
    : base_t(calculateBucketCount((*columns.begin())->getNumCells()),
             Hasher(values), detail::ColumnRowEq(columns)),
    values(columns)
  {
  }

UniqueRowCount::UniqueRowCount(const colvector_t &columns,
                               colvector_t::const_iterator colmask,
                               const HashValues &vals)
    : base_t(calculateBucketCount((*colmask)->getNumCells()),
             Hasher(vals), CEQ(columns, colmask))
  {
  size_t row, numRows = (*colmask)->getNumCells();
  for(row = 0; row < numRows; row++)
    insert(row);
  }

std::ostream &operator<<(std::ostream &s, const UniqueComplement &uc)
  {
  UniqueComplement::const_iterator it;

  for(it = uc.begin(); it != uc.end(); ++it)
    {
    s << it->first << " : ";
    write_set<idvector_t>(s, *(it->second));
    s << "\n";
    }
  return s;
  }
