/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "stdafx.h"
#include <fstream>
#include "hashfunction.hpp"

RandomTable<8>  randomtable;

HashValues::HashValues(const colvector_t &columns, colvector_t::const_iterator colMask)
  {
  colvector_t::const_iterator it = columns.begin(), pend = columns.end();

  allocateTable((*it)->getNumCells()); // All columns have same # of columns
  while(it != pend)
    {
    if (it != colMask)
      {
      (*it)->xorHashValues(m_HashValues);
      }
    ++it;
    }
  }

HashValues::HashValues(const colvector_t &columns)
  {
  colvector_t::const_iterator it = columns.begin(), pend = columns.end();

  allocateTable((*it)->getNumCells()); // All columns have same # of columns
  while(it != pend)
    {
    (*it)->xorHashValues(m_HashValues);
    ++it;
    }
  }

void HashValues::xorColumn(const Column &colMask)
  {
  colMask.xorHashValues(m_HashValues);
  }

HashValues::HashValues(const Column &column)
  {
  allocateTable(column.getNumCells());
  column.xorHashValues(m_HashValues);
  }

void HashValues::allocateTable(size_t tableSize)
  {
  m_HashValues = new size_t[tableSize];
  m_TableSize  = tableSize;
  std::fill(m_HashValues, m_HashValues + tableSize, 0);
  }

void HashValues::dump(const std::string &filename) const
  {
  std::ofstream f(filename.c_str());
  std::copy(m_HashValues, m_HashValues + m_TableSize, std::ostream_iterator<size_t>(f,"\n"));
  }

size_t HashValues::uniqueValues() const
  {
  size_t *tmp = new size_t[m_TableSize];

  std::copy(m_HashValues, m_HashValues + m_TableSize, tmp);
  std::sort(tmp, tmp + m_TableSize);
  size_t *last = std::unique(tmp, tmp + m_TableSize);
  delete[] tmp;
  return std::distance(tmp, last);
  }

HashValues::~HashValues()
  {
  delete[] m_HashValues;
  }

