/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "stdafx.h"
#include <math.h>
#include "variable.hpp"

Variable::Variable(size_t id, const std::string &name, size_t domainsize, bool link_var)
: m_Name(name), m_ValueValid(domainsize, true), m_Id(id), 
  m_DomainSize(domainsize), m_DomainBits(0), m_IsLinkVar(link_var)
  {

  }

unsigned char Variable::getDomainBits() const
  {
  // Cache domain bits
  if (m_DomainBits == 0)
    {
    m_DomainBits = (unsigned char)(ceil(log(m_DomainSize)/log(2.0)));
    }
  return m_DomainBits;
  }

size_t Variable::getValidDomainSize() const
  {
  return std::count(m_ValueValid.begin(), m_ValueValid.end(), true);
  }

void Variable::removeInvalidValues()
  {
  m_ValueValid = std::vector<bool>(getValidDomainSize(), true);
  m_DomainSize = m_ValueValid.size();
  m_DomainBits = 0;
  }

varset_t intersection(const varset_t &u, const varset_t &v)
  {
  varset_t res;
  std::set_intersection(u.begin(), u.end(), v.begin(), v.end(),
    std::insert_iterator<varset_t>(res, res.begin()), VariablePtrOrder());
  return res;
  }

varset_t operator-(const varset_t &u, const varset_t &v)
  {
  varset_t res;
  std::set_difference(u.begin(), u.end(), v.begin(), v.end(),
    std::insert_iterator<varset_t>(res, res.begin()), VariablePtrOrder());
  return res;
  }

varset_t operator+(const varset_t &u, const varset_t &v)
  {
  varset_t res;
  std::set_union(u.begin(), u.end(), v.begin(), v.end(),
    std::insert_iterator<varset_t>(res, res.begin()), VariablePtrOrder());
  return res;
  }

std::ostream &operator<<(std::ostream &s, const Variable &v)
  {
  s << "Var " << v.getName() << " = {";
  for (size_t i = 0; i < v.getDomainSize(); i++)
    {
    if (i > 0)
      s << ",";
    s << (int)i << "=" << v.getDomainValueAsString(i);
    }
  s << "}";
  return s;
  }

std::ostream &operator<<(std::ostream &s, const Variable *v)
  {
  return s << v->getName();
  }
