/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "stdafx.h"
#pragma warning( push )
#pragma warning( disable : 4100 4267 4512 4702)
#include "Arr/arr.h"
#include "Interval/interval.h"
#include "ArrayLib/adbrt.h"
#pragma warning( pop ) 

#include "runtime_network.hpp"
#include "csplib/variable.hpp"
#include "csplib/utility/formatter.hpp"
#include "csplib/adb/adbio.hpp"
#include "adbtest.hpp"

AdbTest::AdbTest(const char *infile1, const char *infile2)
  {
  adb = new AdbRT;
  adb->Load(infile2, true);

  ADB::importADBFile(rn, infile1);

  for(size_t i = 0; i < rn.getNumVariables(); ++i)
    {
    if (rn.getVariable(i).isLinkVar())
      continue;
    char name[8192];
    adb->GetVarName(i, name, sizeof(name));
    if (rn.getVariable(i).getName() != name)
      {
      std::cout << "Var #" << i << " doesn't match. rn=" << rn.getVariable(i).getName() << ", adb=" << name << "\n";
      INTERNAL_ERROR;
      }
    }
  }

AdbTest::~AdbTest()
  {
  delete adb;
  }

void AdbTest::test()
  {
  testTSV();
  testVars();
  }

void AdbTest::checkVar(size_t varnum)
  {
  Variable &v = rn.getVariable(varnum);
  Arr       varvalue = adb->GetVarStatus((int)varnum);
  
  for(size_t i = 0; i < v.getDomainSize(); ++i)
    {
    if ((v.isValueValid(i) == true && varvalue.GetIntElem((int)i) != 3) ||
        (v.isValueValid(i) == false && varvalue.GetIntElem((int)i) == 3))
      {
      std::cout << "Var " << v.getName() << " differ on domain value (" << i << "," << v.getDomainValueAsString(i) << 
        ") rn=" << v.isValueValid(i) << ", adb=" << varvalue.GetIntElem((int)i) << "\n";
      INTERNAL_ERROR;
      }
    }
  }

void AdbTest::checkValue(size_t varnum, size_t domainnum)
  {
    
  bool adbcontract;  
  Arr  VarDecl = adb->GetVarDecl((int)varnum);
  
  cVarTypeType typ = adb->GetVarType(varnum);
  
  Arr values;
  
  if (typ == VTYPE_ENUM || typ == VTYPE_INTV)
    {
    values = Vector(T_PNTR, 1);
    values[0] = VarDecl[(int)domainnum];
    }
  else
    {
    values = Vector(T_LONG, 1);
    values.Set(VarDecl.GetIntElem(domainnum), 0);
    }
  
  adb->SetVarValue((int)varnum, values);
  adb->StateComplete(adbcontract);

  bool contract = !rn.add_constraint(varnum, domainnum);
  
  if (contract != adbcontract)
    {
    std::cout << "Different contradiction state for var (" << varnum << "," << domainnum << 
    ") rn=" << contract << ", adb=" << adbcontract << "\n";
    }

  for(size_t i = 0; i < rn.getNumVariables(); ++i)
    {
    if (!rn.getVariable(i).isLinkVar())
      checkVar(i);
    }
  }

void AdbTest::testTSV()
  {
  Formatter f("Checking initial domain values");

  rn.reset();
  adb->Reset();

  for(size_t i = 0; i < rn.getNumVariables(); ++i)
    {
    f.writeProgress(i+1, rn.getNumVariables()); 
    if (!rn.getVariable(i).isLinkVar())
      checkVar(i);
    }
  }

void AdbTest::testVars()
  {
  Formatter f("Checking variable propagation");
  
  for(size_t i = 0; i < rn.getNumVariables(); ++i)
    {
    Variable &v = rn.getVariable(i);

    f.writeProgress(i+1, rn.getNumVariables(), v.getDomainSize()); 

    if (v.isLinkVar())
      continue;

    for(size_t j = 0; j < v.getDomainSize(); j++)
      {
      rn.reset();
      adb->Reset();

      std::cout << "SetVar " << v.getName() << "=" << v.getDomainValueAsString(j) << "\n";
      if (v.isValueValid(j))
        checkValue(i, j);
      }
    }
  }
