/*
 * Copyright (C) 2003 Jeppe Nejsum Madsen, nejsum@diku.dk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "stdafx.h"
#include "csplib/graph.hpp"
#include "simulatorui.hpp"
#include "runtime_network.hpp"

using namespace boost;

static const bool IGNORE_LINKVARS = true;

SimulatorUI::SimulatorUI(RuntimeNetwork &rn)
: instance(rn)
  {
  rn.reset();
  }

void SimulatorUI::start()
  {
  bool done = false;
  while(!done)
    {
    printDomains();
    std::cout << "\nEnter selection var=value or quit to exit\n";
    std::string input;
    std::cin >> input;
    if (input == "quit")
      done = true;
    else
      {
      size_t pos = input.find("=");
      if (pos == std::string::npos)
        continue;
      std::string var,value;
      var   = std::string(input, 0, pos);
      value = std::string(input, pos+1);
      if (value == "")
        clearConstraint(var);
      else
        addConstraint(var,value);
      }
    }
  }

void SimulatorUI::printDomains()
  {
  std::cout << "Variables and domains:\n";

  for(size_t i = 0; i < instance.getNumVariables(); ++i)
    {
    Variable &v = instance.getVariable(i);

    if (IGNORE_LINKVARS && v.isLinkVar())
      continue;

    std::cout << v.getName() << ": ";
    for(size_t j = 0; j < v.getDomainSize(); ++j)
      {
      std::cout << v.getDomainValueAsString(j);
      if (v.isValueValid(j))
        std::cout << "+";
      else
        std::cout << "-";
      std::cout << " ";
      }
    std::cout << "\n";
    }
  }

      
int SimulatorUI::findVar(const std::string &var)
  {
  for(size_t i = 0; i < instance.getNumVariables(); ++i)
    {
    Variable &v = instance.getVariable(i);
    if (v.getName() == var)
      return (int)i;
    }
  std::cout << "Variable " << var << " not found !!\n";
  return -1;
  }

int SimulatorUI::findDomainVal(Variable &var, const std::string &value)
  {
  for(size_t i = 0; i < var.getDomainSize(); ++i)
    {
    if (var.getDomainValueAsString(i) == value)
      return (int)i;
    }
  std::cout << "Domain value " << value << " not found !!\n";
  return -1;
  }

void SimulatorUI::clearConstraint(const std::string &var)
  {
  std::cout << "Clearing constraint on variable " << var << "\n";

  int varnum;
  varnum = findVar(var);
  if (varnum >= 0)
    instance.remove_constraint(varnum);
  }

void SimulatorUI::addConstraint(const std::string &var, const std::string &value)
  {
  std::cout << "Adding constraint on variable " << var << ", value=" << value << "\n";
  int varnum, domnum;
  varnum = findVar(var);
  if (varnum >= 0)
    {
    Variable &v = instance.getVariable(varnum);
    domnum = findDomainVal(v, value);
    if (domnum >= 0)
      {
      if(!instance.add_constraint(v.getId(), domnum))
        {
        std::cout << "**** contradiction -- Press enter\n";
        std::string d;
        std::cin >> d;
        }
      }
    }
  }
    
