#include "stdafx.h"

#include "test.hpp"

void TestSetup::setUp()
  {
  vars = new LinkVariable *[numVars * numDomains];

  for(int i = 0; i < numDomains; i++)
    {
    for (int j = 0; j < numVars; j++)
      {
      *(vars + i + numDomains*j) = new LinkVariable(i + numDomains*j, boost::lexical_cast<std::string>(char('a'+j)), domainSize[i]);
      instance.addVariable(std::auto_ptr<Variable>(*(vars + i + numDomains*j)));
      }
    }
  }

void TestSetup::tearDown()
  {
  //for(int i = 0; i < numDomains*numVars; i++)
  //  delete *(vars+i);
    
  delete[] vars;
  }

void TestSetup::assertRelsEqual(Relation &r, Relation &s)
  {
  std::stringstream s1, s2;
  s1 << r;
  s2 << s;
  CPPUNIT_ASSERT(s1.str() == s2.str());
  }

void TestSetup::addDigitCol(Relation &r, Variable &var, std::string s)
  {
  UncompressedColumn  *c = UncompressedColumn::createNew(var);

  std::string::iterator it = s.begin(), pend = s.end();


  while (it != pend)
    {
    size_t val = boost::lexical_cast<size_t>(*it);

    size_t factor = var.getDomainSize()/9;
    if (factor > 0)
      val *= factor;

    c->addCell(val);
    ++it;
    }
  r.addColumn(std::auto_ptr<Column>(c));
  }

Relation *TestSetup::addBinRel(size_t id, char *name, Variable *v1, Variable *v2, char *col1, char *col2)
  {
  Relation *r = new Relation(id, name);
  addDigitCol(*r, *v1, col1);
  addDigitCol(*r, *v2, col2);
  instance.addRelation(std::auto_ptr<Relation>(r));
  return r;
  }

const int TestSetup::domainSize[] = {8,15,31,63,120,250,42000,310000};
const int TestSetup::numDomains = sizeof(domainSize)/sizeof(domainSize[0]);
/*

void testMem()
  {
  for(;;)
    {
    Relation  r(0, "exp");
    VarBool a(0,"a"), b(1,"b"), c(2,"c"), d(3,"d");

    addBoolCol(r, a, "000011110011010");
    addBoolCol(r, b, "001100110101001");
    addBoolCol(r, c, "010101011111000");
    addBoolCol(r, d, "111111110000000");
    
//    std::cout << r;

    r.compress();
//    std::cout << r;
    }
  }

void testMem2(const char *infile)
  {
  CSP             instance;

  importADBFile(instance, infile);
  Relation *r = (*instance.getRelations().begin());
  for(;;)
    {
    std::cout << "compressing...\n" << std::flush;
    r->compress();
    std::cout << "uncompressing...\n" << std::flush;
    r->uncompress();
    }
  }

void testJoin()
  {
  CSP             instance;

  importADBFile(instance, "simple.adb");
  CSP::relcol_t::iterator it = instance.getRelations().begin();

  Relation *r1 = *it++;
  Relation *r2 = *it++;

  r1->compress();
  r2->compress();
  std::cout << "r1:\n" << *r1 << "\nr2:\n" << *r2 << "\n" << std::flush;
  Relation *r = r1->join(*r2);

  std::cout << "Result:\n" << *r << "\n" << std::flush;
  }
  */

int main( int argc, char **argv)
{
  CppUnit::TextUi::TestRunner runner;
  CppUnit::TestFactoryRegistry &registry = CppUnit::TestFactoryRegistry::getRegistry();
  runner.addTest(registry.makeTest());   
  return runner.run();
}

