#include "stdafx.h"
#include "test.hpp"

class JoinTest : public TestSetup
  {
  CPPUNIT_TEST_SUITE( JoinTest );
  CPPUNIT_TEST(testJoin1);
  CPPUNIT_TEST(testJoin2);
  CPPUNIT_TEST(testJoinNoCommon);
  CPPUNIT_TEST_SUITE_END();
  public:
    JoinTest() : TestSetup(4) {}

    void testJoin1()
      {
      for(int i = 0; i < numDomains; i++)
        {
        Variable  *a = *(vars+i);
        Variable  *b = *(vars+numDomains+i);

        Relation rel1(0, "rel1");
        addDigitCol(rel1, *a, "0011");
        addDigitCol(rel1, *b, "4567");
        rel1.compress();

        Relation rel2(1, "rel2");
        addDigitCol(rel2, *a, "01120314");
        addDigitCol(rel2, *b, "44556677");
        rel2.compress();
        
        Relation result(2, "result");
        addDigitCol(result, *a, "01");
        addDigitCol(result, *b, "47");

	std::auto_ptr<Relation> joined = rel1.join(rel2,2,100000);
        joined->uncompress();
        std::stringstream s1, s2;
        s1 << *joined;
        s2 << result;
        CPPUNIT_ASSERT(s1.str() == s2.str());
        }
      }

    void testJoin2()
      {
      for(int i = 0; i < numDomains; i++)
        {
        Variable  *a = *(vars+i);
        Variable  *b = *(vars+numDomains+i);

        Relation rel1(0, "rel1");
        addDigitCol(rel1, *a, "4567");
        addDigitCol(rel1, *b, "0011");
        rel1.compress();

        Relation rel2(1, "rel2");
        addDigitCol(rel2, *a, "44556677");
        addDigitCol(rel2, *b, "01120314");
        rel2.compress();
        
        Relation result(2, "result");
        addDigitCol(result, *a, "47");
        addDigitCol(result, *b, "01");

	std::auto_ptr<Relation> joined = rel1.join(rel2,2,10000);
        joined->uncompress();
        std::stringstream s1, s2;
        s1 << *joined;
        s2 << result;
        CPPUNIT_ASSERT(s1.str() == s2.str());
        }
      }

    void testJoinNoCommon()
      {
      for(int i = 0; i < numDomains; i++)
        {
        Variable  *a = *(vars+i);
        Variable  *b = *(vars+numDomains+i);

        Relation rel1(0, "rel1");
        addDigitCol(rel1, *a, "012");
        rel1.compress();

        Relation rel2(1, "rel2");
        addDigitCol(rel2, *b, "345");
        rel2.compress();
        
        Relation result(2, "result");
        addDigitCol(result, *a, "000111222");
        addDigitCol(result, *b, "345345345");

        std::auto_ptr<Relation> joined = rel1.join(rel2,2,10000);
        joined->uncompress();
        std::stringstream s1, s2;
        rel_tuples(s1, *joined);
        rel_tuples(s2, result);
        CPPUNIT_ASSERT(s1.str() == s2.str());
        }
      }

  };

CPPUNIT_TEST_SUITE_REGISTRATION( JoinTest );