#include "stdafx.h"
#include "test.hpp"

class VariableTest : public CppUnit::TestFixture 
  {
  CPPUNIT_TEST_SUITE( VariableTest );
  CPPUNIT_TEST( testVariable );
  CPPUNIT_TEST( testLinkVariable );
  CPPUNIT_TEST_SUITE_END();

  public:
    VariableTest()
    {
    }

    void testVariable()
      {
      ConcreteVariable<int> v1(1, "v1");
      ConcreteVariable<int> v2(2, "v2");

      for(int i = 0; i < 9; i++)
        {
        v1.addDomainValue(i);
        v2.addDomainValue(i*100+1000);
        }

      CPPUNIT_ASSERT(v1.isLinkVar() == false);
      CPPUNIT_ASSERT(v1.getDomainBits() == 4);
      CPPUNIT_ASSERT(v1.getValidDomainSize() == 9);
      CPPUNIT_ASSERT(v1.isValueValid(0));
      CPPUNIT_ASSERT(v1.isValueValid(8));
      CPPUNIT_ASSERT(v1 < v2);
      CPPUNIT_ASSERT(v1.getDomainValueAsString(3) == "3");
      CPPUNIT_ASSERT(v2.getDomainValueAsString(3) == "1300");
      v1.setValueInvalid(3);
      v2.setValueInvalid(3);
      CPPUNIT_ASSERT(v1.getDomainBits() == 4);
      CPPUNIT_ASSERT(v1.getValidDomainSize() == 8);
      v1.removeInvalidValues();
      v2.removeInvalidValues();
      CPPUNIT_ASSERT(v1.getDomainBits() == 3);
      CPPUNIT_ASSERT(v1.getValidDomainSize() == 8);
      CPPUNIT_ASSERT(v1.getDomainSize() == 8);
      CPPUNIT_ASSERT(v1.getDomainValueAsString(3) == "4");
      CPPUNIT_ASSERT(v2.getDomainValueAsString(3) == "1400");
      }

    void testLinkVariable()
      {
      LinkVariable          v3(3, "v3", 10);


      CPPUNIT_ASSERT(v3.isLinkVar());
      CPPUNIT_ASSERT(v3.getDomainBits() == 4);
      CPPUNIT_ASSERT(v3.getValidDomainSize() == 10);
      CPPUNIT_ASSERT(v3.isValueValid(0));
      CPPUNIT_ASSERT(v3.isValueValid(9));

      v3.setValueInvalid(4);
      v3.setValueInvalid(6);
      CPPUNIT_ASSERT(v3.getValidDomainSize() == 8);
      CPPUNIT_ASSERT(v3.getDomainSize() == 10);
      
      CPPUNIT_ASSERT(v3.isValueValid(4) == false);
      CPPUNIT_ASSERT(v3.isValueValid(6) == false);

      v3.removeInvalidValues();
      CPPUNIT_ASSERT(v3.isValueValid(4));
      CPPUNIT_ASSERT(v3.isValueValid(6));
      CPPUNIT_ASSERT(v3.getValidDomainSize() == 8);
      CPPUNIT_ASSERT(v3.getDomainSize() == 8);
      CPPUNIT_ASSERT(v3.getValidDomainSize() == 8);
      CPPUNIT_ASSERT(v3.getDomainBits() == 3);
      }
  };

CPPUNIT_TEST_SUITE_REGISTRATION( VariableTest );
